//  Copyright 2022 Nikita Golikov
 
#include <bits/stdc++.h>
 
using namespace std;
 
using uint = unsigned int;
using ll = long long;
using ull = unsigned long long;
 
#ifdef GOLIKOV
    #define debug(x) cerr << (#x) << ":\t" << (x) << endl
#else
    #define debug(x) 238;
#endif
 
template <class A, class B>
bool smin(A &x, B &&y) {
  if (y < x) {
    x = y;
    return true;
  }
  return false;
}
 
template <class A, class B>
bool smax(A &x, B &&y) {
  if (x < y) {
    x = y;
    return true;
  }
  return false;
}
 
template <class T>
struct Point {
    T x{}, y{};
 
    Point() = default;
 
    Point(T x_, T y_) : x(x_), y(y_) {}
 
    Point operator+(const Point& other) const {
        return Point(x + other.x, y + other.y);
    }
 
    Point operator-(const Point& other) const {
        return Point(x - other.x, y - other.y);
    }
 
    Point operator+() const {
        return *this;
    }
 
    Point operator-() const {
        return Point(-x, -y);
    }
 
    Point operator*(T k) const {
        return Point(x * k, y * k);
    }
 
    Point operator/(T k) const {
        return Point(x / k, y / k);
    }
 
    T operator*(const Point& other) const {
        return x * other.x + y * other.y;
    }
 
    T operator%(const Point& other) const {
        return x * other.y - y * other.x;
    }
 
    T lenSqr() const {
        return x * x + y * y;
    }
 
    double len() const {
        return sqrt(lenSqr());
    }
 
    T distSqr(const Point& other) const {
        return (other - *this).lenSqr();
    }
 
    double dist(const Point& other) const {
        return sqrt(distSqr(other));
    }
 
    Point norm() const {
        return *this / len();
    }
 
    // Counterclockwise rotation by 90 degrees
    Point ort() const {
        return Point(-y, x);
    }
 
    // Counterclockwise rotation by c = cos(alpha), s = sin(alpha)
    Point rot(double c, double s) const {
        return *this * c + ort() * s;
    }
 
    // Counterclockwise rotation by alpha
    Point rot(double alpha) const {
        return rot(cos(alpha), sin(alpha));
    }
 
    double arg() const {
        return atan2(y, x);
    }
 
    friend istream& operator>>(istream& in, Point& p) {
      return in >> p.x >> p.y;
    }
 
    friend ostream& operator<<(ostream& out, Point p) {
      return out << p.x << ' ' << p.y;
    }
 
    bool up() const {
        return y > 0 || (y == 0 && x >= 0);
    }
 
    bool operator==(Point const& other) const {
        return x == other.x && y == other.y;    
    }
 
    bool operator!=(Point const& other) const {
        return x != other.x || y != other.y;    
    }
 
    static auto angleCmp(Point O) {
      return [O](Point x, Point y) {
        x = x - O;
        y = y - O;
        auto d = x.up() - y.up();
        if (d == 0) {
          auto vmul = x % y;
          return vmul == 0 ? x.lenSqr() < y.lenSqr() : vmul > 0;
        }
        return d > 0;
      };
    }
 
    static auto angleCmp() {
      return angleCmp(Point());
    }
 
    bool operator<(Point const& other) const {
        return tie(x, y) < tie(other.x, other.y);
    }
 
    friend bool ccw(Point A, Point B, Point C) {
        return (B - A) % (C - A) >= 0;
    }
 
    //  polygon ccw
    friend bool insidePolygon(vector<Point> const& p, Point O) {
      for (int i = 0; i < int(p.size()); ++i) {
        if (!ccw(O, p[i], p[(i + 1) % int(p.size())])) return false;
      }
      return true;
    }
};
 
string solve(vector<pair<int, int>> aa, int k) {
  int n = int(aa.size());
  using P = Point<ll>;
  vector<P> a(n);
  for (int i = 0; i < n; ++i) {
    a[i] = P(aa[i].first, aa[i].second);
  }
  vector nxt(n, vector<int>(n, -1));
  for (int j = 0; j < n; ++j) {
    vector<pair<P, int>> ord;
    for (int t = 0; t < n; ++t) {
      if (t != j) {
        ord.emplace_back(a[t], t);
      }
    }
    auto cmp = [&](auto x, auto y) {
      return P::angleCmp(a[j])(x.first, y.first);
    };
    sort(ord.begin(), ord.end(), cmp);
    vector<int> go(n - 1);
    for (int t = 1; t < n - 1; ++t) {
      auto x = ord[t].first - a[j];
      auto y = ord[t - 1].first - a[j];
      if (x.up() == y.up() && x % y == 0) {
        go[t] = go[t - 1];
      } else {
        go[t] = t;
      }
    }
    for (int i = 0; i < n; ++i) {
      if (i == j) continue;
      int p = lower_bound(ord.begin(), ord.end(), make_pair(a[j] + a[j] - a[i], -1), cmp) - ord.begin();
      if (p > 0 && (ord[p - 1].first - a[j]).up() == (a[j] - a[i]).up() && (ord[p - 1].first - a[j]) % (a[j] - a[i]) == 0) {
        --p;
      }
      if (p != n - 1) {
        p = go[p];
      }
      nxt[i][j] = ord[(p + k) % (n - 1)].second;
    }
  }
 
  string ans;
  for (int i = 0; i < n; ++i) {
    bool inCycle = false;
    bool was = false;
    for (int j = 0; j < n; ++j) {
      if (i == j) {
        continue;
      }
      int curi = j;
      int curj = nxt[i][j];
      for (int iter = 0; iter < n * n * 2; ++iter) {
        if (curi == i) {
          was = true;
          if (curj == j) {
            inCycle = true;
          }
        }
        int ni = curj;
        int nj = nxt[curi][curj];
        curi = ni;
        curj = nj;
      }
    }
    ans += (inCycle ? "G" : was ? "B" : "R");
  }
  return ans;
}
 
int main() {
#ifdef GOLIKOV
  assert(freopen("in", "rt", stdin));
  auto _clock_start = chrono::high_resolution_clock::now();
#endif
  ios::sync_with_stdio(false);
  cin.tie(nullptr);
 
  int n, k; cin >> n >> k;
  vector<pair<int, int>> a(n);
  for (auto& [x, y] : a) cin >> x >> y;
  cout << solve(a, k - 1) << '\n';
 
#ifdef GOLIKOV
  cerr << "Executed in " << chrono::duration_cast<chrono::milliseconds>(
      chrono::high_resolution_clock::now()
          - _clock_start).count() << "ms." << endl;
#endif
  return 0;
}
