

#include "testlib.h"
#include <iostream>
#include <vector>
#include <algorithm>
#include <map>
#include <string>

using namespace std;

map<string, string> params;

long long get(string name, long long def = 0) {
    if (params.count(name)) {
        return stoll(params[name]);
    } else {
        return def;
    }
}

const int MX = 1'000'000'000;

int builder(vector<int>& vec, int l, int r, int mn = 0) {
    if (l == r) {
        mn++;
        vec[l] = mn;
        return mn;
    } else {
        int mid = (l + r) / 2;
        mn++;
        vec[mid] = mn;
        
        if (l <= mid - 1) {
            mn = builder(vec, l, mid - 1, mn);
        }
        if (mid + 1 <= r) {
            mn = builder(vec, mid + 1, r, mn);
        }
        
        return mn;
    }
}

void gen_numbers(int n) {
    int pmode = get("pmode");

    vector<int> perm(n);
    if (pmode == 1) { // random permutation
        for (int i = 0; i < n; i++)
            perm[i] = i + 1;
        shuffle(perm.begin(), perm.end());
    } else if (pmode == 2) { // some sort of binary tree build
        builder(perm, 0, n - 1);
    } else if (pmode == 3) { // increasing sequence
        for (int i = 0; i < n; i++)
            perm[i] = i + 1;
    } else if (pmode == 4) { // decreasing sequence
        for (int i = 0; i < n; i++)
            perm[i] = n - i;
    } else if (pmode == 5) { // all subpermutations of some sort
        perm.clear();
        vector<int> avperm;
        int factk = get("factk");
        for (int i = 0; i < factk; i++) {
            avperm.push_back(i);
        }
        
        int last = 0;
        while (perm.size() != n) {
            do {
                if (avperm.size() > n - last) {
                    while (last != n) {
                        perm.push_back(last + 1);
                        last++;
                    }
                    break;
                }
                for (auto c : avperm) {
                    perm.push_back(last + c + 1);
                }
                last += avperm.size();
                if (perm.size() == n) break;
            } while (next_permutation(avperm.begin(), avperm.end()));
        }
    }

    for (int i = 0; i < n; i++) {
        cout << perm[i] << " \n"[i == n - 1];
    }
}

void gen_queries(int n, int q) {
    int qmode = get("qmode");
    
    if (qmode == 1) { // random queries
        for (int i = 0; i < q; i++) {
            int a = rnd.next(1, n);
            cout << a << " " << rnd.next(1, n) << "\n";
        }
    } else if (qmode == 2) { // all possible queries
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                cout << i + 1 << " " << j + 1 << "\n";
            }
        }
    } else if (qmode == 3) { // random queries with index \in [imin, imax], k \in [kmin, kmax]
        int imin = get("imin");
        int imax = get("imax");
        int kmin = get("kmin");
        int kmax = get("kmax");
        
        imin = max(imin, 1);
        kmin = max(kmin, 1);
        
        imax = min(imax, n);
        kmax = min(kmax, n);
        
        for (int i = 0; i < q; i++) {
            int a = rnd.next(imin, imax);
            cout << a << " " << rnd.next(kmin, kmax) << "\n";
        }
    } else if (qmode == 4) { // prints all queries starting with smallest k until reaches limit
        int curq = 0;
        for (int j = 0; j < n; j++) {
            for (int i = 0; i < n; i++) {
                curq++;
                cout << i + 1 << " " << j + 1 << "\n";
                if (curq == q) break;
            }
            if (curq == q) break;
        }
    } else if (qmode == 5) { // prints all queries starting with largest k until reaches limit
        int curq = 0;
        for (int i = 0; i < n; i++) {
            for (int j = n - 1; j >= 0; j--) {
                curq++;
                cout << i + 1 << " " << j + 1 << "\n";
                if (curq == q) break;
            }
            if (curq == q) break;
        }
    }
}

void build_test() {
    int n = get("n"), q = get("q");
    cout << n << " " << q << "\n";
    
    gen_numbers(n);
    gen_queries(n, q);
}

int main(int argc, char* argv[])
{
    registerGen(argc, argv, 1);
    for (int i = 1; i < argc; i++) {
        string s = string(argv[i]);
        if (s.find("=") != string::npos) {
            auto pos = s.find("=");
            params[s.substr(0, pos)] = s.substr(pos + 1);
        } else {
            params[s] = "";
        }
    }

    build_test();

    return 0;
}

